@@ -1,4 +1,5 @@
 """CLI parser setup and helpers."""
+
 from __future__ import annotations
 
 import argparse
@@ -7,33 +8,40 @@ import os
 import sys
 from argparse import Namespace
 from pathlib import Path
-from typing import Any, Callable, Sequence
+from typing import TYPE_CHECKING, Any
 
-from ansiblelint.config import DEFAULT_KINDS, DEFAULT_WARN_LIST, PROFILES
-from ansiblelint.constants import (
-    CUSTOM_RULESDIR_ENVVAR,
-    DEFAULT_RULESDIR,
-    INVALID_CONFIG_RC,
+from ansiblelint.config import (
+    DEFAULT_KINDS,
+    DEFAULT_WARN_LIST,
+    PROFILES,
+    Options,
+    log_entries,
 )
+from ansiblelint.constants import CUSTOM_RULESDIR_ENVVAR, DEFAULT_RULESDIR, EPILOG, RC
 from ansiblelint.file_utils import (
     Lintable,
     abspath,
     expand_path_vars,
-    guess_project_dir,
+    find_project_root,
     normpath,
 )
+from ansiblelint.loaders import IGNORE_FILE
 from ansiblelint.schemas.main import validate_file_schema
 from ansiblelint.yaml_utils import clean_json
 
+if TYPE_CHECKING:
+    from collections.abc import Callable, Sequence
+
+
 _logger = logging.getLogger(__name__)
 _PATH_VARS = [
-    "exclude_paths",
     "rulesdir",
 ]
 
 
 def expand_to_normalized_paths(
-    config: dict[str, Any], base_dir: str | None = None
+    config: dict[str, Any],
+    base_dir: str | None = None,
 ) -> None:
     """Mutate given config normalizing any path values in it."""
     # config can be None (-c /dev/null)
@@ -53,35 +61,43 @@ def expand_to_normalized_paths(
         config[paths_var] = normalized_paths
 
 
-def load_config(config_file: str) -> dict[Any, Any]:
+def load_config(config_file: str | None) -> tuple[dict[Any, Any], str | None]:
     """Load configuration from disk."""
     config_path = None
+
+    if config_file == "/dev/null":
+        _logger.debug("Skipping config file as it was set to /dev/null")
+        return {}, config_file
+
     if config_file:
         config_path = os.path.abspath(config_file)
         if not os.path.exists(config_path):
             _logger.error("Config file not found '%s'", config_path)
-            sys.exit(INVALID_CONFIG_RC)
+            sys.exit(RC.INVALID_CONFIG)
     config_path = config_path or get_config_path()
     if not config_path or not os.path.exists(config_path):
         # a missing default config file should not trigger an error
-        return {}
+        return {}, None
 
     config_lintable = Lintable(
-        config_path, kind="ansible-lint-config", base_kind="text/yaml"
+        config_path,
+        kind="ansible-lint-config",
+        base_kind="text/yaml",
     )
 
     for error in validate_file_schema(config_lintable):
         _logger.error("Invalid configuration file %s. %s", config_path, error)
-        sys.exit(INVALID_CONFIG_RC)
+        sys.exit(RC.INVALID_CONFIG)
 
     config = clean_json(config_lintable.data)
     if not isinstance(config, dict):
-        raise RuntimeError("Schema failed to properly validate the config file.")
+        msg = "Schema failed to properly validate the config file."
+        raise TypeError(msg)
     config["config_file"] = config_path
     config_dir = os.path.dirname(config_path)
     expand_to_normalized_paths(config, config_dir)
 
-    return config
+    return config, config_path
 
 
 def get_config_path(config_file: str | None = None) -> str | None:
@@ -89,7 +105,13 @@ def get_config_path(config_file: str | None = None) -> str | None:
     if config_file:
         project_filenames = [config_file]
     else:
-        project_filenames = [".ansible-lint", ".config/ansible-lint.yml"]
+        project_filenames = [
+            ".ansible-lint",
+            ".ansible-lint.yml",
+            ".ansible-lint.yaml",
+            ".config/ansible-lint.yml",
+            ".config/ansible-lint.yaml",
+        ]
     parent = tail = os.getcwd()
     while tail:
         for project_filename in project_filenames:
@@ -115,7 +137,8 @@ class AbspathArgAction(argparse.Action):
         values: str | Sequence[Any] | None,
         option_string: str | None = None,
     ) -> None:
-        if isinstance(values, (str, Path)):
+        _logger.debug(option_string)
+        if isinstance(values, str | Path):
             values = [values]
         if values:
             normalized_values = [
@@ -126,34 +149,37 @@ class AbspathArgAction(argparse.Action):
 
 
 class WriteArgAction(argparse.Action):
-    """Argparse action to handle the --write flag with optional args."""
+    """Argparse action to handle the --fix flag with optional args."""
 
     _default = "__default__"
 
     # noinspection PyShadowingBuiltins
-    def __init__(  # pylint: disable=too-many-arguments,redefined-builtin
+    def __init__(  # pylint: disable=too-many-arguments,redefined-builtin,too-many-positional-arguments
         self,
         option_strings: list[str],
         dest: str,
         nargs: int | str | None = None,
         const: Any = None,
         default: Any = None,
-        type: Callable[[str], Any] | None = None,
+        type: Callable[[str], Any] | None = None,  # noqa: A002
         choices: list[Any] | None = None,
+        *,
         required: bool = False,
-        help: str | None = None,
+        help: str | None = None,  # noqa: A002
         metavar: str | None = None,
     ) -> None:
         """Create the argparse action with WriteArg-specific defaults."""
         if nargs is not None:
-            raise ValueError("nargs for WriteArgAction must not be set.")
+            msg = "nargs for WriteArgAction must not be set."
+            raise ValueError(msg)
         if const is not None:
-            raise ValueError("const for WriteArgAction must not be set.")
+            msg = "const for WriteArgAction must not be set."
+            raise ValueError(msg)
         super().__init__(
             option_strings=option_strings,
             dest=dest,
-            nargs="?",  # either 0 (--write) or 1 (--write=a,b,c) argument
-            const=self._default,  # --write (no option) implicitly stores this
+            nargs="?",  # either 0 (--fix) or 1 (--fix=a,b,c) argument
+            const=self._default,  # --fix (no option) implicitly stores this
             default=default,
             type=type,
             choices=choices,
@@ -172,11 +198,11 @@ class WriteArgAction(argparse.Action):
         lintables = getattr(namespace, "lintables", None)
         if not lintables and isinstance(values, str):
             # args are processed in order.
-            # If --write is after lintables, then that is not ambiguous.
-            # But if --write comes first, then it might actually be a lintable.
+            # If --fix is after lintables, then that is not ambiguous.
+            # But if --fix comes first, then it might actually be a lintable.
             maybe_lintable = Path(values)
             if maybe_lintable.exists():
-                setattr(namespace, "lintables", [values])
+                namespace.lintables = [values]
                 values = []
         if isinstance(values, str):
             values = values.split(",")
@@ -185,28 +211,44 @@ class WriteArgAction(argparse.Action):
         if not values:
             values = previous_values
         elif previous_values != default:
-            values = previous_values + values
+            values = previous_values + values  # type: ignore[operator]
         setattr(namespace, self.dest, values)
 
     @classmethod
-    def merge_write_list_config(
-        cls, from_file: list[str], from_cli: list[str]
+    def merge_fix_list_config(
+        cls,
+        from_file: list[str],
+        from_cli: list[str],
     ) -> list[str]:
-        """Combine the write_list from file config with --write CLI arg.
+        """Determine the write_list value based on cli vs config.
 
-        Handles the implicit "all" when "__default__" is present and file config is empty.
+        When --fix is not passed from command line the from_cli is an empty list,
+        so we use the file.
+
+        When from_cli is not an empty list, we ignore the from_file value.
         """
-        if not from_file or "none" in from_cli:
-            # --write is the same as --write=all
-            return ["all" if value == cls._default else value for value in from_cli]
-        # --write means use the config from the config file
-        from_cli = [value for value in from_cli if value != cls._default]
-        return from_file + from_cli
+        if not from_file:
+            arguments = ["all"] if from_cli == [cls._default] else from_cli
+        else:
+            arguments = from_file
+        for magic_value in ("none", "all"):
+            if magic_value in arguments and len(arguments) > 1:
+                msg = f"When passing '{magic_value}' to '--fix', you cannot pass other values."
+                raise RuntimeError(
+                    msg,
+                )
+        if len(arguments) == 1 and arguments[0] == "none":
+            arguments = []
+        return arguments
 
 
 def get_cli_parser() -> argparse.ArgumentParser:
     """Initialize an argument parser."""
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(
+        epilog=EPILOG,
+        # Avoid rewrapping description and epilog
+        formatter_class=argparse.RawTextHelpFormatter,
+    )
 
     listing_group = parser.add_mutually_exclusive_group()
     listing_group.add_argument(
@@ -215,7 +257,7 @@ def get_cli_parser() -> argparse.ArgumentParser:
         dest="list_profiles",
         default=False,
         action="store_true",
-        help="List all profiles, no formatting options available.",
+        help="List all profiles.",
     )
     listing_group.add_argument(
         "-L",
@@ -223,35 +265,39 @@ def get_cli_parser() -> argparse.ArgumentParser:
         dest="list_rules",
         default=False,
         action="store_true",
-        help="List all the rules. For listing rules only the following formats "
-        "for argument -f are supported: {plain, rich, md}",
+        help="List all the rules.",
     )
     listing_group.add_argument(
         "-T",
         "--list-tags",
         dest="list_tags",
         action="store_true",
-        help="List all the tags and the rules they cover. Increase the verbosity level "
-        "with `-v` to include 'opt-in' tag and its rules.",
+        help="List all the tags and the rules they cover.",
     )
     parser.add_argument(
         "-f",
         "--format",
         dest="format",
-        default="rich",
+        default=None,
         choices=[
-            "rich",
-            "plain",
+            "brief",
+            # "plain",
+            "full",
             "md",
             "json",
             "codeclimate",
             "quiet",
             "pep8",
             "sarif",
-            "docs",  # internally used
         ],
         help="stdout formatting, json being an alias for codeclimate. (default: %(default)s)",
     )
+    parser.add_argument(
+        "--sarif-file",
+        default=None,
+        type=Path,
+        help="SARIF output file",
+    )
     parser.add_argument(
         "-q",
         dest="quiet",
@@ -275,21 +321,12 @@ def get_cli_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="parseable output, same as '-f pep8'",
     )
-    parser.add_argument(
-        "--progressive",
-        dest="progressive",
-        default=False,
-        action="store_true",
-        help="Return success if number of violations compared with"
-        "previous git commit has not increased. This feature works"
-        "only in git repositories.",
-    )
     parser.add_argument(
         "--project-dir",
         dest="project_dir",
-        default=".",
+        default=None,
         help="Location of project/repository, autodetected based on location "
-        " of configuration file.",
+        "of configuration file.",
     )
     parser.add_argument(
         "-r",
@@ -317,23 +354,16 @@ def get_cli_parser() -> argparse.ArgumentParser:
         help="Return non-zero exit code on warnings as well as errors",
     )
     parser.add_argument(
-        "--write",
+        "--fix",
         dest="write_list",
         # this is a tri-state argument that takes an optional comma separated list:
-        #   not provided, --write, --write=a,b,c
         action=WriteArgAction,
-        help="Allow ansible-lint to reformat YAML files and run rule transforms "
-        "(Reformatting YAML files standardizes spacing, quotes, etc. "
-        "A rule transform can fix or simplify fixing issues identified by that rule). "
+        help="Allow ansible-lint to perform auto-fixes, including YAML reformatting. "
         "You can limit the effective rule transforms (the 'write_list') by passing a "
         "keywords 'all' or 'none' or a comma separated list of rule ids or rule tags. "
-        "YAML reformatting happens whenever '--write' or '--write=' is used. "
-        "'--write' and '--write=all' are equivalent: they allow all transforms to run. "
-        "The effective list of transforms comes from 'write_list' in the config file, "
-        "followed whatever '--write' args are provided on the commandline. "
-        "'--write=none' resets the list of transforms to allow reformatting YAML "
-        "without running any of the transforms (ie '--write=none,rule-id' will "
-        "ignore write_list in the config file and only run the rule-id transform).",
+        "YAML reformatting happens whenever '--fix' or '--fix=' is used. "
+        "'--fix' and '--fix=all' are equivalent: they allow all transforms to run. "
+        "Presence of --fix in command overrides config file value.",
     )
     parser.add_argument(
         "--show-relpath",
@@ -363,7 +393,15 @@ def get_cli_parser() -> argparse.ArgumentParser:
         dest="skip_list",
         default=[],
         action="append",
-        help="only check rules whose id/tags do not " "match these values",
+        help="only check rules whose id/tags do not match these values. \
+            e.g: --skip-list=name,run-once",
+    )
+    parser.add_argument(
+        "--generate-ignore",
+        dest="generate_ignore",
+        action="store_true",
+        default=False,
+        help="Generate a text file '.ansible-lint-ignore' that ignores all found violations. Each line contains filename and rule id separated by a space.",
     )
     parser.add_argument(
         "-w",
@@ -399,23 +437,40 @@ def get_cli_parser() -> argparse.ArgumentParser:
     parser.add_argument(
         "--exclude",
         dest="exclude_paths",
-        action=AbspathArgAction,
-        type=Path,
+        action="extend",
+        nargs="+",
+        type=str,
         default=[],
-        help="path to directories or files to skip. " "This option is repeatable.",
+        help="path to directories or files to skip. This option is repeatable.",
     )
     parser.add_argument(
         "-c",
         "--config-file",
         dest="config_file",
-        help="Specify configuration file to use. By default it will look for '.ansible-lint' or '.config/ansible-lint.yml'",
+        help="Specify configuration file to use. By default it will look for '.ansible-lint', '.ansible-lint.yml', '.ansible-lint.yaml', '.config/ansible-lint.yml', or '.config/ansible-lint.yaml'",
+    )
+    parser.add_argument(
+        "-i",
+        "--ignore-file",
+        dest="ignore_file",
+        type=Path,
+        default=None,
+        help=f"Specify ignore file to use. By default it will look for '{IGNORE_FILE.default}' or '{IGNORE_FILE.alternative}'",
+    )
+    parser.add_argument(
+        "-I",
+        "--inventory",
+        dest="inventory",
+        action="append",
+        type=str,
+        help="Specify inventory host path or comma separated host list",
     )
     parser.add_argument(
         "--offline",
         dest="offline",
         action="store_const",
         const=True,
-        help="Disable installation of requirements.yml",
+        help="Disable installation of requirements.yml and schema refreshing",
     )
     parser.add_argument(
         "--version",
@@ -425,14 +480,13 @@ def get_cli_parser() -> argparse.ArgumentParser:
         dest="lintables",
         nargs="*",
         action="extend",
-        help="One or more files or paths. When missing it will "
-        " enable auto-detection mode.",
+        help="One or more files or paths. When missing it will enable auto-detection mode.",
     )
 
     return parser
 
 
-def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespace:
+def merge_config(file_config: dict[Any, Any], cli_config: Options) -> Options:
     """Combine the file config with the CLI args."""
     bools = (
         "display_relative_path",
@@ -440,7 +494,6 @@ def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespac
         "quiet",
         "strict",
         "use_default_rules",
-        "progressive",
         "offline",
     )
     # maps lists to their default config values
@@ -455,13 +508,15 @@ def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespac
         "enable_list": [],
         "only_builtins_allow_collections": [],
         "only_builtins_allow_modules": [],
+        "supported_ansible_also": [],
         # do not include "write_list" here. See special logic below.
     }
 
     scalar_map = {
         "loop_var_prefix": None,
-        "project_dir": ".",
+        "project_dir": None,
         "profile": None,
+        "sarif_file": None,
     }
 
     if not file_config:
@@ -470,6 +525,10 @@ def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespac
         for entry, default in lists_map.items():
             if not getattr(cli_config, entry, None):
                 setattr(cli_config, entry, default)
+        if cli_config.write_list is None:
+            cli_config.write_list = []
+        elif cli_config.write_list == [WriteArgAction._default]:  # noqa: SLF001
+            cli_config.write_list = ["all"]
         return cli_config
 
     for entry in bools:
@@ -477,15 +536,15 @@ def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespac
         v = getattr(cli_config, entry) or file_value
         setattr(cli_config, entry, v)
 
-    for entry, default in scalar_map.items():
-        file_value = file_config.pop(entry, default)
+    for entry, default_scalar in scalar_map.items():
+        file_value = file_config.pop(entry, default_scalar)
         v = getattr(cli_config, entry, None) or file_value
         setattr(cli_config, entry, v)
 
     # if either commandline parameter or config file option is set merge
     # with the other, if neither is set use the default
     for entry, default in lists_map.items():
-        if getattr(cli_config, entry, None) or entry in file_config.keys():
+        if getattr(cli_config, entry, None) or entry in file_config:
             value = getattr(cli_config, entry, [])
             value.extend(file_config.pop(entry, []))
         else:
@@ -497,7 +556,7 @@ def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespac
     setattr(
         cli_config,
         entry,
-        WriteArgAction.merge_write_list_config(
+        WriteArgAction.merge_fix_list_config(
             from_file=file_config.pop(entry, []),
             from_cli=getattr(cli_config, entry, []) or [],
         ),
@@ -513,41 +572,66 @@ def merge_config(file_config: dict[Any, Any], cli_config: Namespace) -> Namespac
     # append default kinds to the custom list
     kinds = file_config.get("kinds", [])
     kinds.extend(DEFAULT_KINDS)
-    setattr(cli_config, "kinds", kinds)
+    cli_config.kinds = kinds
 
     return cli_config
 
 
-def get_config(arguments: list[str]) -> Namespace:
+def get_config(arguments: list[str]) -> Options:
     """Extract the config based on given args."""
     parser = get_cli_parser()
-    options = parser.parse_args(arguments)
+    # translate deprecated options
+    for i, value in enumerate(arguments):
+        if arguments[i].startswith("--write"):
+            arguments[i] = value.replace("--write", "--fix")
+            _logger.warning(
+                "Replaced deprecated '--write' option with '--fix', change you call to avoid future regressions when we remove old option.",
+            )
+    options = Options(**vars(parser.parse_args(arguments)))
 
     # docs is not document, being used for internal documentation building
-    if options.list_rules and options.format not in ["plain", "rich", "md", "docs"]:
+    if options.list_rules and options.format not in [
+        None,
+        "brief",
+        "full",
+        "md",
+    ]:
         parser.error(
             f"argument -f: invalid choice: '{options.format}'. "
-            f"In combination with argument -L only 'plain', "
-            f"'rich' or 'md' are supported with -f."
+            f"In combination with argument -L only 'brief', "
+            f"'rich' or 'md' are supported with -f.",
         )
 
     # save info about custom config file, as options.config_file may be modified by merge_config
-    has_custom_config = not options.config_file
-
-    file_config = load_config(options.config_file)
-
+    file_config, options.config_file = load_config(options.config_file)
     config = merge_config(file_config, options)
 
-    options.rulesdirs = get_rules_dirs(options.rulesdir, options.use_default_rules)
+    options.rulesdirs = get_rules_dirs(
+        options.rulesdir,
+        use_default=options.use_default_rules,
+    )
 
-    if has_custom_config and options.project_dir == ".":
-        project_dir = guess_project_dir(options.config_file)
+    if not options.project_dir:
+        project_dir, method = find_project_root(
+            srcs=options.lintables,
+            config_file=options.config_file,
+        )
         options.project_dir = os.path.expanduser(normpath(project_dir))
+        log_entries.append(
+            (
+                logging.INFO,
+                f"Identified [repr.path]{project_dir}[/] as project root due [bold]{method}[/].",
+            ),
+        )
 
     if not options.project_dir or not os.path.exists(options.project_dir):
-        raise RuntimeError(
-            f"Failed to determine a valid project_dir: {options.project_dir}"
-        )
+        msg = f"Failed to determine a valid project_dir: {options.project_dir}"
+        raise RuntimeError(msg)
+
+    # expand user home dir in exclude_paths
+    options.exclude_paths = [
+        os.path.expandvars(os.path.expanduser(p)) for p in options.exclude_paths
+    ]
 
     # Compute final verbosity level by subtracting -q counter.
     options.verbosity -= options.quiet
@@ -559,11 +643,12 @@ def print_help(file: Any = sys.stdout) -> None:
     get_cli_parser().print_help(file=file)
 
 
-def get_rules_dirs(rulesdir: list[str], use_default: bool = True) -> list[str]:
+def get_rules_dirs(rulesdir: list[Path], *, use_default: bool = True) -> list[Path]:
     """Return a list of rules dirs."""
     default_ruledirs = [DEFAULT_RULESDIR]
     default_custom_rulesdir = os.environ.get(
-        CUSTOM_RULESDIR_ENVVAR, os.path.join(DEFAULT_RULESDIR, "custom")
+        CUSTOM_RULESDIR_ENVVAR,
+        os.path.join(DEFAULT_RULESDIR, "custom"),
     )
     custom_ruledirs = sorted(
         str(x.resolve())
@@ -571,7 +656,11 @@ def get_rules_dirs(rulesdir: list[str], use_default: bool = True) -> list[str]:
         if x.is_dir() and (x / "__init__.py").exists()
     )
 
+    result: list[Any] = []
     if use_default:
-        return rulesdir + custom_ruledirs + default_ruledirs
-
-    return rulesdir or custom_ruledirs + default_ruledirs
+        result = rulesdir + custom_ruledirs + default_ruledirs
+    elif rulesdir:
+        result = rulesdir
+    else:
+        result = custom_ruledirs + default_ruledirs
+    return [Path(p) for p in result]
