@@ -10,7 +10,7 @@ import time
 import urllib.request
 import warnings
 from dataclasses import dataclass, field
-from functools import lru_cache
+from http.client import HTTPException
 from importlib.metadata import PackageNotFoundError, distribution, version
 from pathlib import Path
 from typing import Any
@@ -42,7 +42,8 @@ DEFAULT_KINDS = [
     {"text": "**/templates/**/*.*"},  # templates are likely not validable
     {"execution-environment": "**/execution-environment.yml"},
     {"ansible-lint-config": "**/.ansible-lint"},
-    {"ansible-lint-config": "**/.config/ansible-lint.yml"},
+    {"ansible-lint-config": "**/.ansible-lint.{yaml,yml}"},
+    {"ansible-lint-config": "**/.config/ansible-lint.{yaml,yml}"},
     {"ansible-navigator-config": "**/ansible-navigator.{yaml,yml}"},
     {"inventory": "**/inventory/**.{yaml,yml}"},
     {"requirements": "**/meta/requirements.{yaml,yml}"},  # v1 only
@@ -172,6 +173,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
     version: bool = False  # display version command
     list_profiles: bool = False  # display profiles command
     ignore_file: Path | None = None
+    inventory: list[str] | None = None
     max_tasks: int = 100
     max_block_depth: int = 20
     # Refer to https://docs.ansible.com/ansible/latest/reference_appendices/release_and_maintenance.html#ansible-core-support-matrix
@@ -200,26 +202,10 @@ options = Options()
 # Used to store detected tag deprecations
 used_old_tags: dict[str, str] = {}
 
-# Used to store collection list paths (with mock paths if needed)
-collection_list: list[str] = []
-
 # Used to store log messages before logging is initialized (level, message)
 log_entries: list[tuple[int, str]] = []
 
 
-@lru_cache
-def ansible_collections_path() -> str:
-    """Return collection path variable for current version of Ansible."""
-    # respect Ansible behavior, which is to load old name if present
-    for env_var in [
-        "ANSIBLE_COLLECTIONS_PATHS",
-        "ANSIBLE_COLLECTIONS_PATH",
-    ]:  # pragma: no cover
-        if env_var in os.environ:
-            return env_var
-    return "ANSIBLE_COLLECTIONS_PATH"
-
-
 def in_venv() -> bool:
     """Determine whether Python is running from a venv."""
     if hasattr(sys, "real_prefix") or os.environ.get("CONDA_EXE", None) is not None:
@@ -237,7 +223,7 @@ def guess_install_method() -> str:
         if (distribution(package_name).read_text("INSTALLER") or "").strip() != "pip":
             return ""
     except PackageNotFoundError as exc:
-        logging.debug(exc)
+        _logger.debug(exc)
         return ""
 
     pip = ""
@@ -267,16 +253,16 @@ def guess_install_method() -> str:
 
             dist = get_default_environment().get_distribution(package_name)
             if dist:
-                logging.debug("Found %s dist", dist)
+                _logger.debug("Found %s dist", dist)
                 for _ in uninstallation_paths(dist):
                     use_pip = True
             else:
-                logging.debug("Skipping %s as it is not installed.", package_name)
+                _logger.debug("Skipping %s as it is not installed.", package_name)
                 use_pip = False
     except (AttributeError, ModuleNotFoundError) as exc:
         # On Fedora 36, we got a AttributeError exception from pip that we want to avoid
         # On NixOS, we got a ModuleNotFoundError exception from pip that we want to avoid
-        logging.debug(exc)
+        _logger.debug(exc)
         use_pip = False
 
     # We only want to recommend pip for upgrade if it looks safe to do so.
@@ -321,7 +307,7 @@ def get_version_warning() -> str:
         with open(cache_file, encoding="utf-8") as f:
             data = json.load(f)
 
-    if refresh or not data:
+    if not options.offline and (refresh or not data):
         release_url = (
             "https://api.github.com/repos/ansible/ansible-lint/releases/latest"
         )
@@ -330,7 +316,7 @@ def get_version_warning() -> str:
                 data = json.load(url)
                 with open(cache_file, "w", encoding="utf-8") as f:
                     json.dump(data, f)
-        except (URLError, HTTPError) as exc:  # pragma: no cover
+        except (URLError, HTTPError, HTTPException) as exc:  # pragma: no cover
             _logger.debug(
                 "Unable to fetch latest version from %s due to: %s",
                 release_url,
@@ -338,13 +324,14 @@ def get_version_warning() -> str:
             )
             return ""
 
-    html_url = data["html_url"]
-    new_version = Version(data["tag_name"][1:])  # removing v prefix from tag
+    if data:
+        html_url = data["html_url"]
+        new_version = Version(data["tag_name"][1:])  # removing v prefix from tag
 
-    if current_version > new_version:
-        msg = "[dim]You are using a pre-release version of ansible-lint.[/]"
-    elif current_version < new_version:
-        msg = f"""[warning]A new release of ansible-lint is available: [red]{current_version}[/] → [green][link={html_url}]{new_version}[/][/][/]"""
-        msg += f" Upgrade by running: [info]{pip}[/]"
+        if current_version > new_version:
+            msg = "[dim]You are using a pre-release version of ansible-lint.[/]"
+        elif current_version < new_version:
+            msg = f"""[warning]A new release of ansible-lint is available: [warning]{current_version}[/] → [success][link={html_url}]{new_version}[/link][/][/]"""
+            msg += f" Upgrade by running: [info]{pip}[/]"
 
     return msg
