@@ -1,4 +1,5 @@
 """All internal ansible-lint rules."""
+
 from __future__ import annotations
 
 import copy
@@ -6,14 +7,11 @@ import inspect
 import logging
 import re
 import sys
-from argparse import Namespace
 from collections import defaultdict
-from functools import lru_cache
+from collections.abc import Iterable, Iterator, MutableMapping, MutableSequence
 from importlib import import_module
 from pathlib import Path
-from typing import Any, Iterable, Iterator, MutableMapping, MutableSequence, cast
-
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
+from typing import TYPE_CHECKING, Any, cast
 
 import ansiblelint.skip_utils
 import ansiblelint.utils
@@ -25,12 +23,18 @@ from ansiblelint._internal.rules import (
     RuntimeErrorRule,
     WarningRule,
 )
-from ansiblelint.config import PROFILES, get_rule_config
+from ansiblelint.app import App, get_app
+from ansiblelint.config import PROFILES, Options
 from ansiblelint.config import options as default_options
 from ansiblelint.constants import LINE_NUMBER_KEY, RULE_DOC_URL, SKIPPED_RULES_KEY
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, expand_paths_vars
 
+if TYPE_CHECKING:
+    from ruamel.yaml.comments import CommentedMap, CommentedSeq
+
+    from ansiblelint.errors import RuleMatchTransformMeta
+
 _logger = logging.getLogger(__name__)
 
 match_types = {
@@ -52,12 +56,6 @@ class AnsibleLintRule(BaseRule):
         """Return rule documentation url."""
         return RULE_DOC_URL + self.id + "/"
 
-    @property
-    def rule_config(self) -> dict[str, Any]:
-        """Retrieve rule specific configuration."""
-        return get_rule_config(self.id)
-
-    @lru_cache(maxsize=256)
     def get_config(self, key: str) -> Any:
         """Return a configured value for given key string."""
         return self.rule_config.get(key, None)
@@ -70,31 +68,32 @@ class AnsibleLintRule(BaseRule):
         text = re.sub(r"{#.+?#}", "JINJA_COMMENT", text)
         return text
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def create_matcherror(
         self,
-        message: str | None = None,
-        linenumber: int = 1,
+        message: str = "",
+        lineno: int = 1,
         details: str = "",
         filename: Lintable | None = None,
         tag: str = "",
+        transform_meta: RuleMatchTransformMeta | None = None,
     ) -> MatchError:
         """Instantiate a new MatchError."""
         match = MatchError(
             message=message,
-            linenumber=linenumber,
+            lineno=lineno,
             details=details,
-            filename=filename,
+            lintable=filename or Lintable(""),
             rule=copy.copy(self),
+            tag=tag,
+            transform_meta=transform_meta,
         )
-        if tag:
-            match.tag = tag
         # search through callers to find one of the match* methods
         frame = inspect.currentframe()
         match_type: str | None = None
         while not match_type and frame is not None:
             func_name = frame.f_code.co_name
-            match_type = match_types.get(func_name, None)
+            match_type = match_types.get(func_name)
             if match_type:
                 # add the match_type to the match
                 match.match_type = match_type
@@ -104,43 +103,46 @@ class AnsibleLintRule(BaseRule):
 
     @staticmethod
     def _enrich_matcherror_with_task_details(
-        match: MatchError, task: dict[str, Any]
+        match: MatchError,
+        task: ansiblelint.utils.Task,
     ) -> None:
         match.task = task
         if not match.details:
             match.details = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
-        if match.linenumber < task[LINE_NUMBER_KEY]:
-            match.linenumber = task[LINE_NUMBER_KEY]
+
+        match.lineno = max(match.lineno, task[LINE_NUMBER_KEY])
 
     def matchlines(self, file: Lintable) -> list[MatchError]:
         matches: list[MatchError] = []
         # arrays are 0-based, line numbers are 1-based
         # so use prev_line_no as the counter
-        for (prev_line_no, line) in enumerate(file.content.split("\n")):
+        for prev_line_no, line in enumerate(file.content.split("\n")):
             if line.lstrip().startswith("#"):
                 continue
 
-            rule_id_list = ansiblelint.skip_utils.get_rule_skips_from_line(line)
+            rule_id_list = ansiblelint.skip_utils.get_rule_skips_from_line(
+                line,
+                lintable=file,
+            )
             if self.id in rule_id_list:
                 continue
 
             result = self.match(line)
             if not result:
                 continue
-            message = None
+            message = ""
             if isinstance(result, str):
                 message = result
             matcherror = self.create_matcherror(
                 message=message,
-                linenumber=prev_line_no + 1,
+                lineno=prev_line_no + 1,
                 details=line,
                 filename=file,
             )
             matches.append(matcherror)
         return matches
 
-    # pylint: disable=too-many-branches
-    def matchtasks(self, file: Lintable) -> list[MatchError]:  # noqa: C901
+    def matchtasks(self, file: Lintable) -> list[MatchError]:
         """Call matchtask for each task inside file and return aggregate results.
 
         Most rules will never need to override matchtasks because its main
@@ -154,48 +156,55 @@ class AnsibleLintRule(BaseRule):
         ):
             return matches
 
-        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file)
-        for raw_task, task, skipped_tags, error in tasks_iterator:
-            if error is not None:
+        for task in ansiblelint.utils.task_in_list(
+            data=file.data,
+            kind=file.kind,
+            file=file,
+        ):
+            if task.error is not None:
                 # normalize_task converts AnsibleParserError to MatchError
-                return [error]
+                return [task.error]
 
             if (
-                self.id in skipped_tags
-                or ("action" not in task)
-                or "skip_ansible_lint" in task.get("tags", [])
+                self.id in task.skip_tags
+                or ("action" not in task.normalized_task)
+                or "skip_ansible_lint" in task.normalized_task.get("tags", [])
             ):
                 continue
 
             if self.needs_raw_task:
-                task["__raw_task__"] = raw_task
+                task.normalized_task["__raw_task__"] = task.raw_task
 
             result = self.matchtask(task, file=file)
             if not result:
                 continue
 
             if isinstance(result, Iterable) and not isinstance(
-                result, str
+                result,
+                str,
             ):  # list[MatchError]
                 # https://github.com/PyCQA/pylint/issues/6044
                 # pylint: disable=not-an-iterable
                 for match in result:
-                    if match.tag in skipped_tags:
+                    if match.tag in task.skip_tags:
                         continue
-                    self._enrich_matcherror_with_task_details(match, task)
+                    self._enrich_matcherror_with_task_details(
+                        match,
+                        task,
+                    )
                     matches.append(match)
                 continue
             if isinstance(result, MatchError):
-                if result.tag in skipped_tags:
+                if result.tag in task.skip_tags:
                     continue
                 match = result
             else:  # bool or string
-                message = None
+                message = ""
                 if isinstance(result, str):
                     message = result
                 match = self.create_matcherror(
                     message=message,
-                    linenumber=task[LINE_NUMBER_KEY],
+                    lineno=task.normalized_task[LINE_NUMBER_KEY],
                     filename=file,
                 )
 
@@ -215,7 +224,16 @@ class AnsibleLintRule(BaseRule):
         if isinstance(yaml, str):
             if yaml.startswith("$ANSIBLE_VAULT"):
                 return []
-            return [MatchError(filename=file, rule=LoadingFailureRule())]
+            if self._collection is None:
+                msg = f"Rule {self.id} was not added to a collection."
+                raise RuntimeError(msg)
+            return [
+                # pylint: disable=E1136
+                MatchError(
+                    lintable=file,
+                    rule=self._collection["load-failure"],
+                ),
+            ]
         if not yaml:
             return matches
 
@@ -223,7 +241,6 @@ class AnsibleLintRule(BaseRule):
             yaml = [yaml]
 
         for play in yaml:
-
             # Bug #849
             if play is None:
                 continue
@@ -242,7 +259,7 @@ class AnsibleLintRule(BaseRule):
 class TransformMixin:
     """A mixin for AnsibleLintRule to enable transforming files.
 
-    If ansible-lint is started with the ``--write`` option, then the ``Transformer``
+    If ansible-lint is started with the ``--fix`` option, then the ``Transformer``
     will call the ``transform()`` method for every MatchError identified if the rule
     that identified it subclasses this ``TransformMixin``. Only the rule that identified
     a MatchError can do transforms to fix that match.
@@ -309,23 +326,21 @@ class TransformMixin:
         for segment in yaml_path:
             # The cast() calls tell mypy what types we expect.
             # Essentially this does:
-            #   target = target[segment]
             if isinstance(segment, str):
-                target = cast(MutableMapping[str, Any], target)[segment]
+                target = cast("MutableMapping[str, Any]", target)[segment]
             elif isinstance(segment, int):
-                target = cast(MutableSequence[Any], target)[segment]
+                target = cast("MutableSequence[Any]", target)[segment]
         return target
 
 
-# pylint: disable=too-many-nested-blocks
-def load_plugins(  # noqa: max-complexity: 11
+def load_plugins(
     dirs: list[str],
 ) -> Iterator[AnsibleLintRule]:
     """Yield a rule class."""
 
     def all_subclasses(cls: type) -> set[type]:
         return set(cls.__subclasses__()).union(
-            [s for c in cls.__subclasses__() for s in all_subclasses(c)]
+            [s for c in cls.__subclasses__() for s in all_subclasses(c)],
         )
 
     orig_sys_path = sys.path.copy()
@@ -337,11 +352,7 @@ def load_plugins(  # noqa: max-complexity: 11
         # load all modules in the directory
         for f in Path(directory).glob("*.py"):
             if "__" not in f.stem and f.stem not in "conftest":
-                try:
-                    import_module(f"{f.stem}")
-
-                except ImportError as exc:
-                    _logger.warning("Ignore loading rule from %s due to %s", f, exc)
+                import_module(f"{f.stem}")
     # restore sys.path
     sys.path = orig_sys_path
 
@@ -351,12 +362,15 @@ def load_plugins(  # noqa: max-complexity: 11
         # or rules that do not have a valid id. For example, during testing
         # python may load other rule classes, some outside the tested rule
         # directories.
-        if getattr(rule, "id") and Path(inspect.getfile(rule)).parent.absolute() in [
-            Path(x).absolute() for x in dirs
-        ]:
-            if issubclass(rule, BaseRule) and rule.id not in rules:
-                rules[rule.id] = rule()
-    for rule in rules.values():  # type: ignore
+        if (
+            rule.id  # type: ignore[attr-defined]
+            and Path(inspect.getfile(rule)).parent.absolute()
+            in [Path(x).absolute() for x in dirs]
+            and issubclass(rule, BaseRule)
+            and rule.id not in rules
+        ):
+            rules[rule.id] = rule()
+    for rule in rules.values():  # type: ignore[assignment]
         if isinstance(rule, AnsibleLintRule) and bool(rule.id):
             yield rule
 
@@ -364,21 +378,30 @@ def load_plugins(  # noqa: max-complexity: 11
 class RulesCollection:
     """Container for a collection of rules."""
 
-    def __init__(
+    def __init__(  # pylint: disable=too-many-arguments
         self,
-        rulesdirs: list[str] | None = None,
-        options: Namespace = default_options,
+        rulesdirs: list[str] | list[Path] | None = None,
+        options: Options | None = None,
         profile_name: str | None = None,
+        *,
         conditional: bool = True,
+        app: App | None = None,
     ) -> None:
         """Initialize a RulesCollection instance."""
-        self.options = options
+        if options is None:
+            self.options = copy.deepcopy(default_options)
+            # When initialized without options argument we want it to always
+            # be offline as this is done only during testing.
+            self.options.offline = True
+        else:
+            self.options = options
         self.profile = []
+        self.app = app or get_app(cached=True)
+
         if profile_name:
             self.profile = PROFILES[profile_name]
-        if rulesdirs is None:
-            rulesdirs = []
-        self.rulesdirs = expand_paths_vars(rulesdirs)
+        rulesdirs_str = [] if rulesdirs is None else [str(r) for r in rulesdirs]
+        self.rulesdirs = expand_paths_vars(rulesdirs_str)
         self.rules: list[BaseRule] = []
         # internal rules included in order to expose them for docs as they are
         # not directly loaded by our rule loader.
@@ -388,20 +411,24 @@ class RulesCollection:
                 AnsibleParserErrorRule(),
                 LoadingFailureRule(),
                 WarningRule(),
-            ]
+            ],
         )
-        for rule in load_plugins(rulesdirs):
+        for rule in self.rules:
+            rule._collection = self  # noqa: SLF001
+        for rule in load_plugins(rulesdirs_str):
             self.register(rule, conditional=conditional)
         self.rules = sorted(self.rules)
 
-        # when we have a profile we unload some of the rules
-        if profile_name:
+        # When we have a profile we unload some of the rules
+        # But we do include all rules when listing all rules or tags
+        if profile_name and not (self.options.list_rules or self.options.list_tags):
             filter_rules_with_profile(self.rules, profile_name)
 
-    def register(self, obj: AnsibleLintRule, conditional: bool = False) -> None:
+    def register(self, obj: AnsibleLintRule, *, conditional: bool = False) -> None:
         """Register a rule."""
         # We skip opt-in rules which were not manually enabled.
         # But we do include opt-in rules when listing all rules or tags
+        obj._collection = self  # pylint: disable=protected-access # noqa: SLF001
         if any(
             [
                 not conditional,
@@ -410,9 +437,8 @@ class RulesCollection:
                 obj.id in self.options.enable_list,
                 self.options.list_rules,
                 self.options.list_tags,
-            ]
+            ],
         ):
-            obj._collection = self  # pylint: disable=protected-access
             self.rules.append(obj)
 
     def __iter__(self) -> Iterator[BaseRule]:
@@ -427,11 +453,22 @@ class RulesCollection:
         """Return the length of the RulesCollection data."""
         return len(self.rules)
 
+    def __getitem__(self, item: Any) -> BaseRule:
+        """Return a rule from inside the collection based on its id."""
+        if not isinstance(item, str):
+            msg = f"Expected str but got {type(item)} when trying to access rule by it's id"
+            raise TypeError(msg)
+        for rule in self.rules:
+            if rule.id == item:
+                return rule
+        msg = f"Rule {item} is not present inside this collection."
+        raise ValueError(msg)
+
     def extend(self, more: list[AnsibleLintRule]) -> None:
         """Combine rules."""
         self.rules.extend(more)
 
-    def run(  # noqa: max-complexity: 12
+    def run(
         self,
         file: Lintable,
         tags: set[str] | None = None,
@@ -452,10 +489,10 @@ class RulesCollection:
                 return [
                     MatchError(
                         message=str(exc),
-                        filename=file,
-                        rule=LoadingFailureRule(),
+                        lintable=file,
+                        rule=self["load-failure"],
                         tag=f"{LoadingFailureRule.id}[{exc.__class__.__name__.lower()}]",
-                    )
+                    ),
                 ]
 
         for rule in self.rules:
@@ -466,10 +503,18 @@ class RulesCollection:
                 or rule.has_dynamic_tags
                 or not set(rule.tags).union([rule.id]).isdisjoint(tags)
             ):
-                rule_definition = set(rule.tags)
-                rule_definition.add(rule.id)
-                if set(rule_definition).isdisjoint(skip_list):
-                    matches.extend(rule.getmatches(file))
+                if tags and set(rule.tags).union(list(rule.ids().keys())).isdisjoint(
+                    tags,
+                ):
+                    _logger.debug("Skipping rule %s", rule.id)
+                else:
+                    _logger.debug("Running rule %s", rule.id)
+                    rule_definition = set(rule.tags)
+                    rule_definition.add(rule.id)
+                    if set(rule_definition).isdisjoint(skip_list):
+                        matches.extend(rule.getmatches(file))
+            else:
+                _logger.debug("Skipping rule %s", rule.id)
 
         # some rules can produce matches with tags that are inside our
         # skip_list, so we need to cleanse the matches
@@ -480,9 +525,17 @@ class RulesCollection:
     def __repr__(self) -> str:
         """Return a RulesCollection instance representation."""
         return "\n".join(
-            [rule.verbose() for rule in sorted(self.rules, key=lambda x: x.id)]
+            [rule.verbose() for rule in sorted(self.rules, key=lambda x: x.id)],
         )
 
+    def known_tags(self) -> list[str]:
+        """Return a list of known tags, without returning no sub-tags."""
+        tags = set()
+        for rule in self.rules:
+            tags.add(rule.id)
+            tags.update(rule.tags)
+        return sorted(tags)
+
     def list_tags(self) -> str:
         """Return a string with all the tags in the RulesCollection."""
         tag_desc = {
@@ -496,6 +549,7 @@ class RulesCollection:
             "metadata": "Invalid metadata, likely related to galaxy, collections or roles",
             "opt-in": "Rules that are not used unless manually added to `enable_list`",
             "security": "Rules related o potentially security issues, like exposing credentials",
+            "syntax": "Related to wrong or deprecated syntax",
             "unpredictability": "Warn about code that might not work in a predictable way",
             "unskippable": "Indicate a fatal error that cannot be ignored or disabled",
             "yaml": "External linter which will also produce its own rule codes",
@@ -503,17 +557,20 @@ class RulesCollection:
 
         tags = defaultdict(list)
         for rule in self.rules:
+            # Fail early if a rule does not have any of our required tags
+            if not set(rule.tags).intersection(tag_desc.keys()):
+                msg = f"Rule {rule} does not have any of the required tags: {', '.join(tag_desc.keys())}"
+                raise RuntimeError(msg)
             for tag in rule.tags:
-                tags[tag].append(rule.id)
+                tags[tag] = list(rule.ids())
         result = "# List of tags and rules they cover\n"
         for tag in sorted(tags):
-            desc = tag_desc.get(tag, None)
+            desc = tag_desc.get(tag)
             if desc:
                 result += f"{tag}:  # {desc}\n"
             else:
                 result += f"{tag}:\n"
-            # result += f"  rules:\n"
-            for name in tags[tag]:
+            for name in sorted(tags[tag]):
                 result += f"  - {name}\n"
         return result
 
@@ -529,6 +586,8 @@ def filter_rules_with_profile(rule_col: list[BaseRule], profile: str) -> None:
             included.add(rule)
         extends = PROFILES[extends].get("extends", None)
     for rule in rule_col.copy():
+        if rule.unloadable:
+            continue
         if rule.id not in included:
             _logger.debug(
                 "Unloading %s rule due to not being part of %s profile.",
@@ -536,18 +595,4 @@ def filter_rules_with_profile(rule_col: list[BaseRule], profile: str) -> None:
                 profile,
             )
             rule_col.remove(rule)
-        else:
-            for tag in ("opt-in", "experimental"):
-                if tag in rule.tags:
-                    _logger.debug(
-                        "Removing tag `%s` from `%s` rule because `%s` profile makes it mandatory.",
-                        tag,
-                        rule.id,
-                        profile,
-                    )
-                    rule.tags.remove(tag)
-                    # rule_col.rules.remove(rule)
-                    # break
-            if "opt-in" in rule.tags:
-                rule.tags.remove("opt-in")
     _logger.debug("%s/%s rules included in the profile", len(rule_col), total_rules)
