@@ -1,11 +1,13 @@
 """Text utils."""
+
 from __future__ import annotations
 
 import re
-from functools import lru_cache
+from functools import cache
 
 RE_HAS_JINJA = re.compile(r"{[{%#].*[%#}]}", re.DOTALL)
-RE_HAS_GLOB = re.compile("[][*?]")
+RE_HAS_GLOB = re.compile(r"[][*?]")
+RE_IS_FQCN_OR_NAME = re.compile(r"^\w+(\.\w+\.\w+)?$")
 
 
 def strip_ansi_escape(data: str | bytes) -> str:
@@ -24,27 +26,39 @@ def toidentifier(text: str) -> str:
     """Convert unsafe chars to ones allowed in variables."""
     result = re.sub(r"[\s-]+", "_", text)
     if not result.isidentifier():
-        raise RuntimeError(
-            f"Unable to convert role name '{text}' to valid variable name."
-        )
+        msg = f"Unable to convert role name '{text}' to valid variable name."
+        raise RuntimeError(msg)
     return result
 
 
 # https://www.python.org/dev/peps/pep-0616/
-def removeprefix(self: str, prefix: str) -> str:
+def removeprefix(text: str, prefix: str) -> str:
     """Remove prefix from string."""
-    if self.startswith(prefix):
-        return self[len(prefix) :]
-    return self[:]
+    if text.startswith(prefix):
+        return text[len(prefix) :]
+    return text[:]
 
 
-@lru_cache(maxsize=None)
+@cache
 def has_jinja(value: str) -> bool:
     """Return true if a string seems to contain jinja templating."""
     return bool(isinstance(value, str) and RE_HAS_JINJA.search(value))
 
 
-@lru_cache(maxsize=None)
+@cache
 def has_glob(value: str) -> bool:
     """Return true if a string looks like having a glob pattern."""
     return bool(isinstance(value, str) and RE_HAS_GLOB.search(value))
+
+
+@cache
+def is_fqcn_or_name(value: str) -> bool:
+    """Return true if a string seems to be a module/filter old name or a fully qualified one."""
+    return bool(isinstance(value, str) and RE_IS_FQCN_OR_NAME.search(value))
+
+
+@cache
+def is_fqcn(value: str) -> bool:
+    """Return true if a string seems to be a fully qualified collection name."""
+    match = RE_IS_FQCN_OR_NAME.search(value)
+    return bool(isinstance(value, str) and match and match.group(1))
